import * as Express from 'express';

import {database, Task, TASKS} from "./Database.js";

function Tasks(request, response, next) {
    database.all('SELECT * FROM ' + TASKS + ' WHERE owner_id = ?', [
        request.user.id
    ], (error, rows) => {
        if (error) return next(error);
        const tasks = rows.map((row: Task) => {
            return {
                id: row.id,
                title: row.title,
                completed: row.completed === 1 ? true : false,
                url: '/' + row.id
            }
        });
        response.locals.tasks = tasks;
        response.locals.activeCount = tasks.filter(task => !task.completed).length;
        response.locals.completedCount = tasks.length - response.locals.activeCount;
        next();
    });
}

export const router = Express.Router();

/* GET home page. */
router.get('/', (request, response, next) => {
    if (!request.user) return response.render('Home');
    next();
}, Tasks, (request, response, next) => {
    response.locals.filter = null;
    response.render('Index', {user: request.user});
});

router.get('/active', Tasks, (request, response, next) => {
    response.locals.tasks = response.locals.tasks.filter(task => !task.completed);
    response.locals.filter = 'active';
    response.render('Index', {user: request.user});
});

router.get('/completed', Tasks, (request, response, next) => {
    response.locals.tasks = response.locals.tasks.filter(task => task.completed);
    response.locals.filter = 'completed';
    response.render('Index', {user: request.user});
});

router.post('/', (request, response, next) => {
    request.body.title = request.body.title.trim();
    next();
}, (request, response, next) => {
    if (request.body.title !== '') return next();
    return response.redirect('/' + (request.body.filter || ''));
}, (request, response, next) => {
    database.run('INSERT INTO ' + TASKS + ' (owner_id, title, completed) VALUES (?, ?, ?)', [
        request.user.id,
        request.body.title,
        request.body.completed == true ? 1 : null
    ], error => {
        if (error) return next(error);
        return response.redirect('/' + (request.body.filter || ''));
    });
});

router.post('/:id(\\d+)', (request, response, next) => {
    request.body.title = request.body.title.trim();
    next();
}, (request, response, next) => {
    if (request.body.title !== '') return next();
    database.run('DELETE FROM ' + TASKS + ' WHERE id = ? AND owner_id = ?', [
        request.params.id,
        request.user.id
    ], error => {
        if (error) return next(error);
        return response.redirect('/' + (request.body.filter || ''));
    });
}, (request, response, next) => {
    database.run('UPDATE ' + TASKS + ' SET title = ?, completed = ? WHERE id = ? AND owner_id = ?', [
        request.body.title,
        request.body.completed !== undefined ? 1 : null,
        request.params.id,
        request.user.id
    ], error => {
        if (error) return next(error);
        return response.redirect('/' + (request.body.filter || ''));
    });
});

router.post('/:id(\\d+)/delete', (request, response, next) => {
    database.run('DELETE FROM ' + TASKS + ' WHERE id = ? AND owner_id = ?', [
        request.params.id,
        request.user.id
    ], error => {
        if (error) return next(error);
        return response.redirect('/' + (request.body.filter || ''));
    });
});

router.post('/toggle-all', (request, response, next) => {
    database.run('UPDATE ' + TASKS + ' SET completed = ? WHERE owner_id = ?', [
        request.body.completed !== undefined ? 1 : null,
        request.user.id
    ], error => {
        if (error) return next(error);
        return response.redirect('/' + (request.body.filter || ''));
    });
});

router.post('/clear-completed', (request, response, next) => {
    database.run('DELETE FROM ' + TASKS + ' WHERE owner_id = ? AND completed = ?', [
        request.user.id,
        1
    ], error => {
        if (error) return next(error);
        return response.redirect('/' + (request.body.filter || ''));
    });
});
